@@ -26,9 +26,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
 #include "xalloc.h"
 #include "xvasprintf.h"
-#include "memcmp2.h"
 #include "minmax.h"
 
 #include "astr.h"
@@ -53,17 +55,17 @@ astr_new (void)
   as->maxlen = ALLOCATION_CHUNK_SIZE;
   as->len = 0;
   as->text = (char *) xzalloc (as->maxlen + 1);
-  memset (as->text, 0, as->maxlen + 1);
   return as;
 }
 
-void
-astr_delete (astr as)
+castr
+castr_new_nstr (const char *s, size_t n)
 {
-  assert (as != NULL);
-  free (as->text);
-  as->text = NULL;
-  free (as);
+  astr as;
+  as = (astr) XZALLOC (struct astr);
+  as->len = n;
+  as->text = s;
+  return as;
 }
 
 const char *
@@ -78,39 +80,79 @@ astr_len (castr as)
   return as->len;
 }
 
-astr
-astr_ncat_cstr (astr as, const char *s, size_t csize)
+static
+void astr_set_len (astr as, size_t len)
 {
-  assert (as != NULL);
-  if (as->len + csize > as->maxlen)
+  if (len > as->maxlen || len < as->maxlen / 2)
     {
-      as->maxlen = as->len + csize + ALLOCATION_CHUNK_SIZE;
+      as->maxlen = len + ALLOCATION_CHUNK_SIZE;
       as->text = xrealloc (as->text, as->maxlen + 1);
     }
-  memcpy (as->text + as->len, s, csize);
-  as->len += csize;
+  as->len = len;
   as->text[as->len] = '\0';
+}
+
+astr
+astr_cat_nstr (astr as, const char *s, size_t csize)
+{
+  assert (as != NULL);
+  size_t oldlen = as->len;
+  astr_set_len (as, as->len + csize);
+  memmove (as->text + oldlen, s, csize);
   return as;
 }
 
 astr
-astr_nreplace_cstr (astr as, size_t pos, size_t size, const char *s,
-                    size_t csize)
+astr_replace_nstr (astr as, size_t pos, const char *s, size_t size)
 {
-  astr tail;
-
   assert (as != NULL);
   assert (pos <= as->len);
-  if (as->len - pos < size)
-    size = as->len - pos;
-  tail = astr_substr (as, pos + (size_t) size,
-                      astr_len (as) - (pos + size));
-  as->len = pos;
-  as->text[pos] = '\0';
-  astr_ncat_cstr (as, s, csize);
-  astr_cat (as, tail);
-  astr_delete (tail);
+  assert (size <= as->len - pos);
+  memmove (as->text + pos, s, size);
+  return as;
+}
 
+astr
+astr_remove (astr as, size_t pos, size_t size)
+{
+  assert (as != NULL);
+  assert (pos <= as->len);
+  assert (size <= as->len - pos);
+  memmove (as->text + pos, as->text + pos + size, as->len - (pos + size));
+  astr_set_len (as, as->len - size);
+  return as;
+}
+
+astr
+astr_insert (astr as, size_t pos, size_t size)
+{
+  assert (as != NULL);
+  assert (pos <= as->len);
+  assert (pos + size >= MAX (pos, size));    /* Check for overflow. */
+  astr_set_len (as, as->len + size);
+  memmove (as->text + pos + size, as->text + pos, as->len - (pos + size));
+  memset (as->text + pos, '\0', size);
+  return as;
+}
+
+astr
+astr_move (astr as, size_t to, size_t from, size_t n)
+{
+  assert (as != NULL);
+  assert (to <= as->len);
+  assert (from <= as->len);
+  assert (n <= as->len - MAX (from, to));
+  memmove (as->text + to, as->text + from, n);
+  return as;
+}
+
+astr
+astr_set (astr as, size_t pos, int c, size_t n)
+{
+  assert (as != NULL);
+  assert (pos <= as->len);
+  assert (n <= as->len - pos);
+  memset (as->text + pos, c, n);
   return as;
 }
 
@@ -120,7 +162,7 @@ astr_nreplace_cstr (astr as, size_t pos, size_t size, const char *s,
  */
 
 char
-astr_get (astr as, size_t pos)
+astr_get (castr as, size_t pos)
 {
   assert (pos <= astr_len (as));
   return (astr_cstr (as))[pos];
@@ -130,7 +172,7 @@ static astr
 astr_ncpy_cstr (astr as, const char *s, size_t len)
 {
   astr_truncate (as, 0);
-  return astr_ncat_cstr (as, s, len);
+  return astr_cat_nstr (as, s, len);
 }
 
 astr
@@ -140,7 +182,7 @@ astr_new_cstr (const char *s)
 }
 
 astr
-astr_cpy (astr as, astr src)
+astr_cpy (astr as, castr src)
 {
   return astr_ncpy_cstr (as, astr_cstr (src), astr_len (src));
 }
@@ -154,52 +196,27 @@ astr_cpy_cstr (astr as, const char *s)
 astr
 astr_cat (astr as, castr src)
 {
-  return astr_ncat_cstr (as, astr_cstr (src), astr_len (src));
+  return astr_cat_nstr (as, astr_cstr (src), astr_len (src));
 }
 
 astr
 astr_cat_cstr (astr as, const char *s)
 {
-  return astr_ncat_cstr (as, s, strlen (s));
+  return astr_cat_nstr (as, s, strlen (s));
 }
 
 astr
 astr_cat_char (astr as, int c)
 {
-  return astr_insert_char (as, astr_len (as), c);
+  char c2 = c;
+  return astr_cat_nstr (as, &c2, 1);
 }
 
 astr
-astr_substr (astr as, size_t pos, size_t size)
+astr_substr (castr as, size_t pos, size_t size)
 {
   assert (pos + size <= astr_len (as));
-  return astr_ncat_cstr (astr_new (), astr_cstr (as) + pos, size);
-}
-
-int
-astr_cmp (astr as1, astr as2)
-{
-  return memcmp2 (astr_cstr (as1), astr_len (as1), astr_cstr (as2), astr_len (as2));
-}
-
-astr
-astr_replace_cstr (astr as, size_t pos, size_t size, const char *s)
-{
-  assert (s != NULL);
-  return astr_nreplace_cstr (as, pos, size, s, strlen (s));
-}
-
-astr
-astr_insert_char (astr as, size_t pos, int c)
-{
-  char ch = (char) c;
-  return astr_nreplace_cstr (as, pos, (size_t) 0, &ch, (size_t) 1);
-}
-
-astr
-astr_remove (astr as, size_t pos, size_t size)
-{
-  return astr_nreplace_cstr (as, pos, size, "", (size_t) 0);
+  return astr_cat_nstr (astr_new (), astr_cstr (as) + pos, size);
 }
 
 astr
@@ -208,51 +225,57 @@ astr_truncate (astr as, size_t pos)
   return astr_remove (as, pos, astr_len (as) - pos);
 }
 
-astr astr_fread (FILE * fp)
+astr
+astr_readf (const char *filename)
 {
-  int c;
-  astr as = astr_new ();
-
-  while ((c = getc (fp)) != EOF)
-    astr_cat_char (as, c);
-
+  astr as = NULL;
+  struct stat st;
+  if (stat (filename, &st) == 0)
+    {
+      size_t size = st.st_size;
+      int fd = open (filename, O_RDONLY);
+      if (fd >= 0)
+        {
+          char buf[BUFSIZ];
+          as = astr_new ();
+          while ((size = read (fd, buf, BUFSIZ)) > 0)
+            astr_cat_nstr (as, buf, size);
+          close (fd);
+        }
+    }
   return as;
 }
 
 astr
-astr_afmt (astr as, const char *fmt, ...)
+astr_vfmt (const char *fmt, va_list ap)
+{
+  return astr_new_cstr (xvasprintf (fmt, ap));
+}
+
+astr
+astr_fmt (const char *fmt, ...)
 {
   va_list ap;
-  char *buf;
   va_start (ap, fmt);
-  buf = xvasprintf (fmt, ap);
-  astr_cat_cstr (as, buf);
-  free (buf);
+  astr as = astr_vfmt (fmt, ap);
   va_end (ap);
   return as;
 }
 
-/*
- * Recase str according to newcase.
- */
 astr
 astr_recase (astr as, enum casing newcase)
 {
   astr bs = astr_new ();
-  size_t i, len;
 
   if (newcase == case_capitalized || newcase == case_upper)
     astr_cat_char (bs, toupper (astr_get (as, 0)));
   else
     astr_cat_char (bs, tolower (astr_get (as, 0)));
 
-  for (i = 1, len = astr_len (as); i < len; i++)
+  for (size_t i = 1, len = astr_len (as); i < len; i++)
     astr_cat_char (bs, ((newcase == case_upper) ? toupper : tolower) (astr_get (as, i)));
 
-  astr_cpy (as, bs);
-  astr_delete (bs);
-
-  return as;
+  return astr_cpy (as, bs);
 }
 
 
@@ -268,7 +291,7 @@ astr_recase (astr as, enum casing newcase)
 static void
 assert_eq (astr as, const char *s)
 {
-  if (STRNEQ (astr_cstr (as), s))
+  if (!STREQ (astr_cstr (as), s))
     {
       printf ("test failed: \"%s\" != \"%s\"\n", astr_cstr (as), s);
       exit (EXIT_FAILURE);
@@ -276,7 +299,7 @@ assert_eq (astr as, const char *s)
 }
 
 /* Stub to make xalloc_die happy. */
-void zile_exit (int doabort) _GL_ATTRIBUTE_NORETURN;
+void _Noreturn zile_exit (int doabort);
 
 void
 zile_exit (int doabort _GL_UNUSED_PARAMETER)
@@ -306,59 +329,23 @@ main (int argc _GL_UNUSED_PARAMETER, char **argv)
   astr_cat_char (as2, '.');
   assert_eq (as2, "The world.");
 
-  astr_delete (as3);
   as3 = astr_substr (as1, astr_len (as1) - 6, 5);
   assert_eq (as3, "world");
 
-  astr_cpy_cstr (as1, "12345");
-  astr_delete (as2);
-
-  astr_cpy_cstr (as1, "12345");
-  astr_insert_char (as1, astr_len (as1) - 2, 'x');
-  astr_insert_char (as1, astr_len (as1) - 6, 'y');
-  astr_insert_char (as1, 7, 'z');
-  assert_eq (as1, "y123x45z");
-
   astr_cpy_cstr (as1, "1234567");
-  astr_replace_cstr (as1, astr_len (as1) - 4, 2, "foo");
-  assert_eq (as1, "123foo67");
-
-  astr_cpy_cstr (as1, "1234567");
-  astr_replace_cstr (as1, 1, 3, "foo");
+  astr_replace_nstr (as1, 1, "foo", 3);
   assert_eq (as1, "1foo567");
 
-  astr_cpy_cstr (as1, "1234567");
-  astr_replace_cstr (as1, astr_len (as1) - 1, 5, "foo");
-  assert_eq (as1, "123456foo");
-
-  astr_cpy_cstr (as1, "1234567");
-  astr_remove (as1, 4, 10);
-  assert_eq (as1, "1234");
-
   astr_cpy_cstr (as1, "12345");
   as2 = astr_substr (as1, astr_len (as1) - 2, 2);
   assert_eq (as2, "45");
 
   astr_cpy_cstr (as1, "12345");
-  astr_delete (as2);
   as2 = astr_substr (as1, astr_len (as1) - 5, 5);
   assert_eq (as2, "12345");
 
-  astr_cpy_cstr (as1, "1234567");
-  astr_replace_cstr (as1, astr_len (as1) - 4, 2, "foo");
-  assert_eq (as1, "123foo67");
-
-  astr_cpy_cstr (as1, "1234567");
-  astr_replace_cstr (as1, 1, 3, "foo");
-  assert_eq (as1, "1foo567");
-
-  astr_cpy_cstr (as1, "1234567");
-  astr_replace_cstr (as1, astr_len (as1) - 1, 5, "foo");
-  assert_eq (as1, "123456foo");
-
-  astr_cpy_cstr (as1, "");
-  astr_afmt (as1, "%s * %d = ", "5", 3);
-  astr_afmt (as1, "%d", 15);
+  as1 = astr_fmt ("%s * %d = ", "5", 3);
+  astr_cat (as1, astr_fmt ("%d", 15));
   assert_eq (as1, "5 * 3 = 15");
 
   astr_cpy_cstr (as1, "some text");
@@ -369,9 +356,6 @@ main (int argc _GL_UNUSED_PARAMETER, char **argv)
   astr_recase (as1, case_lower);
   assert_eq (as1, "some text");
 
-  astr_delete (as1);
-  astr_delete (as2);
-  astr_delete (as3);
   printf ("astr test successful.\n");
 
   return EXIT_SUCCESS;
