@@ -1,6 +1,6 @@
 /* Completion facility functions
 
-   Copyright (c) 1997-2004, 2008-2010, 2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2004, 2008-2011 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -84,7 +84,7 @@ completion_new (int fileflag)
 
   cp->completions = gl_list_create_empty (GL_LINKED_LIST,
                                           completion_STREQ, NULL,
-                                          (gl_listelement_dispose_fn) free, false);
+                                          NULL, false);
   cp->matches = gl_list_create_empty (GL_LINKED_LIST,
                                       completion_STREQ, NULL,
                                       NULL, false);
@@ -98,35 +98,21 @@ completion_new (int fileflag)
   return cp;
 }
 
-/*
- * Dispose an completion structure.
- */
-void
-free_completion (Completion * cp)
-{
-  gl_list_free (cp->completions);
-  gl_list_free (cp->matches);
-  if (cp->flags & CFLAG_FILENAME)
-    astr_delete (cp->path);
-  free (cp->match);
-  free (cp);
-}
-
 /*
  * Scroll completions up.
  */
 void
 completion_scroll_up (void)
 {
-  Window *wp, *old_wp = cur_wp;
-  Point pt;
-
-  wp = find_window ("*Completions*");
+  Window *old_wp = cur_wp;
+  Window *wp = find_window ("*Completions*");
   assert (wp != NULL);
   set_current_window (wp);
-  pt = get_buffer_pt (cur_bp);
-  if (pt.n >= get_buffer_last_line (cur_bp) - get_window_eheight (cur_wp) || !FUNCALL (scroll_up))
-    gotobob ();
+  if (FUNCALL (scroll_up) == leNIL)
+    {
+      FUNCALL (beginning_of_buffer);
+      resync_redisplay (cur_wp);
+    }
   set_current_window (old_wp);
 
   term_redisplay ();
@@ -138,16 +124,13 @@ completion_scroll_up (void)
 void
 completion_scroll_down (void)
 {
-  Window *wp, *old_wp = cur_wp;
-  Point pt;
-
-  wp = find_window ("*Completions*");
+  Window *old_wp = cur_wp;
+  Window *wp = find_window ("*Completions*");
   assert (wp != NULL);
   set_current_window (wp);
-  pt = get_buffer_pt (cur_bp);
-  if (pt.n == 0 || !FUNCALL (scroll_down))
+  if (FUNCALL (scroll_down) == leNIL)
     {
-      gotoeob ();
+      FUNCALL (end_of_buffer);
       resync_redisplay (cur_wp);
     }
   set_current_window (old_wp);
@@ -159,70 +142,46 @@ completion_scroll_down (void)
  * Calculate the maximum length of the completions.
  */
 static size_t
-calculate_max_length (gl_list_t l, size_t size)
+calculate_max_length (gl_list_t l)
 {
-  size_t i, maxlen = 0;
-
-  for (i = 0; i < MIN (size, gl_list_size (l)); i++)
-    {
-      size_t len = strlen ((const char *) gl_list_get_at (l, i));
-      maxlen = MAX (len, maxlen);
-    }
-
+  size_t maxlen = 0;
+  for (size_t i = 0; i < gl_list_size (l); i++)
+    maxlen = MAX (strlen ((const char *) gl_list_get_at (l, i)), maxlen);
   return maxlen;
 }
 
 /*
  * Print the list of completions in a set of columns.
  */
-static void
-completion_print (gl_list_t l, size_t size)
-{
-  size_t i, j, col, max, numcols;
-
-  max = calculate_max_length (l, size) + 5;
-  numcols = (get_window_ewidth (cur_wp) - 1) / max;
-
-  bprintf ("Possible completions are:\n");
-  for (i = col = 0; i < MIN (size, gl_list_size (l)); i++)
-    {
-      const char *s = (const char *) gl_list_get_at (l, i);
-      size_t len = strlen (s);
-      if (col >= numcols)
-        {
-          col = 0;
-          insert_newline ();
-        }
-      insert_nstring (s, len);
-      for (j = max - len; j > 0; --j)
-        insert_char_in_insert_mode (' ');
-      ++col;
-    }
-}
-
 static void
 write_completion (va_list ap)
 {
-  Completion *cp = va_arg (ap, Completion *);
-  int allflag = va_arg (ap, int);
-  size_t num = va_arg (ap, size_t);
-  if (allflag)
-    completion_print (cp->completions, gl_list_size (cp->completions));
-  else
-    completion_print (cp->matches, num);
+  gl_list_t l = va_arg (ap, gl_list_t);
+  size_t max = calculate_max_length (l) + 5;
+  size_t numcols = (get_window_ewidth (cur_wp) - 1) / max;
+
+  bprintf ("Possible completions are:\n");
+  for (size_t i = 0, col = 0; i < gl_list_size (l); i++)
+    {
+      bprintf ("%-*s", max, (const char *) gl_list_get_at (l, i));
+
+      col = (col + 1) % numcols;
+      if (col == 0)
+        insert_newline ();
+    }
 }
 
 /*
  * Popup the completion window.
  */
 static void
-popup_completion (Completion * cp, int allflag, size_t num)
+popup_completion (Completion * cp, int allflag)
 {
   cp->flags |= CFLAG_POPPEDUP;
   if (get_window_next (head_wp) == NULL)
     cp->flags |= CFLAG_CLOSE;
 
-  write_temp_buffer ("*Completions*", true, write_completion, cp, allflag, num);
+  write_temp_buffer ("*Completions*", true, write_completion, allflag ? cp->completions : cp->matches);
 
   if (!(cp->flags & CFLAG_CLOSE))
     cp->old_bp = cur_bp;
@@ -234,81 +193,49 @@ popup_completion (Completion * cp, int allflag, size_t num)
  * Reread directory for completions.
  */
 static int
-completion_readdir (Completion * cp, astr as)
+completion_readdir (Completion * cp, astr path)
 {
-  DIR *dir;
-  char *s1, *s2;
-  const char *pdir;
-  char *base;
-  struct dirent *d;
-  struct stat st;
-  astr bs;
-
-  gl_list_free (cp->completions);
-
   cp->completions = gl_list_create_empty (GL_LINKED_LIST,
                                           completion_STREQ, NULL,
-                                          (gl_listelement_dispose_fn) free, false);
+                                          NULL, false);
 
-  if (!expand_path (as))
+  if (!expand_path (path))
     return false;
 
-  bs = astr_new ();
-
   /* Split up path with dirname and basename, unless it ends in `/',
-     in which case it's considered to be entirely dirname */
-  s1 = xstrdup (astr_cstr (as));
-  s2 = xstrdup (astr_cstr (as));
-  if (astr_get (as, astr_len (as) - 1) != '/')
+     in which case it's considered to be entirely dirname. */
+  astr pdir;
+  if (astr_get (path, astr_len (path) - 1) != '/')
     {
-      char *s = dir_name (s1);
-      /* Append `/' to dir */
-      astr_cat_cstr (bs, s);
-      if (astr_get (bs, astr_len (bs) - 1) != '/')
-        astr_cat_char (bs, '/');
-      free (s);
-      pdir = astr_cstr (bs);
-      base = base_name (s2);
+      pdir = astr_new_cstr (dir_name (astr_cstr (path)));
+      if (!STREQ (astr_cstr (pdir), "/"))
+        astr_cat_char (pdir, '/');
+      astr_cpy_cstr (path, base_name (astr_cstr (path)));
     }
   else
     {
-      pdir = s1;
-      base = xstrdup ("");
+      pdir = astr_cpy (astr_new (), path);
+      astr_truncate (path, 0);
     }
 
-  astr_cpy_cstr (as, base);
-  free (base);
-
-  dir = opendir (pdir);
+  DIR *dir = opendir (astr_cstr (pdir));
   if (dir != NULL)
     {
-      astr buf = astr_new ();
-      while ((d = readdir (dir)) != NULL)
+      for (struct dirent *d = readdir (dir); d != NULL; d = readdir (dir))
         {
-          astr_cpy_cstr (buf, pdir);
-          astr_cat_cstr (buf, d->d_name);
-          if (stat (astr_cstr (buf), &st) != -1)
-            {
-              astr_cpy_cstr (buf, d->d_name);
-              if (S_ISDIR (st.st_mode))
-                astr_cat_char (buf, '/');
-            }
-          else
-            astr_cpy_cstr (buf, d->d_name);
+          astr as = astr_new_cstr (d->d_name);
+          struct stat st;
+          if (stat (xasprintf ("%s%s", astr_cstr (pdir), d->d_name), &st) == 0 &&
+              S_ISDIR (st.st_mode))
+            astr_cat_char (as, '/');
           gl_sortedlist_add (cp->completions, completion_strcmp,
-                             xstrdup (astr_cstr (buf)));
+                             xstrdup (astr_cstr (as)));
         }
       closedir (dir);
 
-      astr_delete (cp->path);
-      cp->path = compact_path (astr_new_cstr (pdir));
-      astr_delete (buf);
+      cp->path = compact_path (pdir);
     }
 
-  astr_delete (bs);
-  free (s1);
-  free (s2);
-
   return dir != NULL;
 }
 
@@ -318,26 +245,20 @@ completion_readdir (Completion * cp, astr as)
 int
 completion_try (Completion * cp, astr search, int popup_when_complete)
 {
-  size_t i, j, ssize;
-  size_t fullmatches = 0, partmatches = 0;
-  char c;
-
-  gl_list_free (cp->matches);
   cp->matches = gl_list_create_empty (GL_LINKED_LIST, completion_STREQ, NULL, NULL, false);
 
   if (cp->flags & CFLAG_FILENAME)
     if (!completion_readdir (cp, search))
       return COMPLETION_NOTMATCHED;
 
-  ssize = astr_len (search);
-
+  size_t ssize = astr_len (search);
   if (ssize == 0)
     {
       cp->match = xstrdup ((const char *) gl_list_get_at (cp->completions, 0));
       if (gl_list_size (cp->completions) > 1)
         {
           cp->matchsize = 0;
-          popup_completion (cp, true, 0);
+          popup_completion (cp, true);
           return COMPLETION_NONUNIQUE;
         }
       else
@@ -347,48 +268,45 @@ completion_try (Completion * cp, astr search, int popup_when_complete)
         }
     }
 
-  for (i = 0; i < gl_list_size (cp->completions); i++)
+  size_t fullmatches = 0;
+  for (size_t i = 0; i < gl_list_size (cp->completions); i++)
     {
       const char *s = (const char *) gl_list_get_at (cp->completions, i);
       if (!strncmp (s, astr_cstr (search), ssize))
         {
-          ++partmatches;
           gl_sortedlist_add (cp->matches, completion_strcmp, s);
           if (STREQ (s, astr_cstr (search)))
             ++fullmatches;
         }
     }
 
-  if (partmatches == 0)
+  if (gl_list_size (cp->matches) == 0)
     return COMPLETION_NOTMATCHED;
-  else if (partmatches == 1)
+  else if (gl_list_size (cp->matches) == 1)
     {
       cp->match = xstrdup ((const char *) gl_list_get_at (cp->matches, 0));
       cp->matchsize = strlen (cp->match);
       return COMPLETION_MATCHED;
     }
-  else if (partmatches > 1 && fullmatches == 1)
+  else if (gl_list_size (cp->matches) > 1 && fullmatches == 1)
     {
       cp->match = xstrdup ((const char *) gl_list_get_at (cp->matches, 0));
       cp->matchsize = strlen (cp->match);
       if (popup_when_complete)
-        popup_completion (cp, false, partmatches);
+        popup_completion (cp, false);
       return COMPLETION_MATCHEDNONUNIQUE;
     }
 
-  for (j = ssize;; ++j)
+  for (size_t j = ssize;; ++j)
     {
-      const char *s = (const char *) gl_list_get_at (cp->matches, 0);
-
-      c = s[j];
-      for (i = 1; i < partmatches; ++i)
+      char c = ((const char *) gl_list_get_at (cp->matches, 0))[j];
+      for (size_t i = 1; i < gl_list_size (cp->matches); ++i)
         {
-          s = gl_list_get_at (cp->matches, i);
-          if (s[j] != c)
+          if (((const char *)(gl_list_get_at (cp->matches, i)))[j] != c)
             {
               cp->match = xstrdup ((const char *) gl_list_get_at (cp->matches, 0));
               cp->matchsize = j;
-              popup_completion (cp, false, partmatches);
+              popup_completion (cp, false);
               return COMPLETION_NONUNIQUE;
             }
         }
