@@ -34,7 +34,7 @@ size_t do_binding_completion (astr as);
 gl_list_t get_key_sequence (void);
 Function get_function_by_keys (gl_list_t keys);
 le *call_command (Function f, int uniarg, bool uniflag, le *branch);
-void process_command (void);
+void get_and_run_command (void);
 void init_default_bindings (void);
 
 /* buffer.c --------------------------------------------------------------- */
@@ -168,9 +168,9 @@ const char *previous_history_element (History * hp);
 const char *next_history_element (History * hp);
 
 /* keycode.c -------------------------------------------------------------- */
-astr chordtostr (size_t key);
+astr chordtodesc (size_t key);
 gl_list_t keystrtovec (const char *key);
-astr keyvectostr (gl_list_t keys);
+astr keyvectodesc (gl_list_t keys);
 
 /* line.c ----------------------------------------------------------------- */
 bool insert_newline (void);
