@@ -21,8 +21,6 @@
 
 #include <config.h>
 
-#include <stdlib.h>
-
 #include "main.h"
 #include "extern.h"
 
@@ -37,28 +35,6 @@ leNew (const char *text)
   return new;
 }
 
-static void
-leReallyWipe (le * list)
-{
-  if (list)
-    {
-      /* free descendants */
-      leWipe (list->branch);
-      leWipe (list->next);
-
-      /* free ourself */
-      free (list->data);
-      free (list);
-    }
-}
-
-void
-leWipe (le * list)
-{
-  if (list && list != leNIL && list != leT)
-    leReallyWipe (list);
-}
-
 static le *
 leAddTail (le * list, le * element)
 {
