@@ -46,12 +46,6 @@ struct Window
 #include "window.h"
 #undef FIELD
 
-static Window *
-window_new (void)
-{
-  return (Window *) XZALLOC (Window);
-}
-
 /*
  * Set the current window and his buffer as the current buffer.
  */
@@ -60,22 +54,19 @@ set_current_window (Window * wp)
 {
   /* Save buffer's point in a new marker.  */
   if (cur_wp->saved_pt)
-    free_marker (cur_wp->saved_pt);
+    unchain_marker (cur_wp->saved_pt);
 
   cur_wp->saved_pt = point_marker ();
 
-  /* Change the current window.  */
   cur_wp = wp;
-
-  /* Change the current buffer.  */
   cur_bp = wp->bp;
 
   /* Update the buffer point with the window's saved point
      marker.  */
   if (cur_wp->saved_pt)
     {
-      set_buffer_pt (cur_bp, get_marker_pt (cur_wp->saved_pt));
-      free_marker (cur_wp->saved_pt);
+      goto_offset (get_marker_o (cur_wp->saved_pt));
+      unchain_marker (cur_wp->saved_pt);
       cur_wp->saved_pt = NULL;
     }
 }
@@ -86,29 +77,25 @@ Split current window into two windows, one above the other.
 Both windows display the same buffer now current.
 +*/
 {
-  Window *newwp;
-
   /* Windows smaller than 4 lines cannot be split. */
   if (cur_wp->fheight < 4)
     {
-      minibuf_error ("Window height %d too small for splitting",
+      minibuf_error ("Window height %d too small (after splitting)",
                      cur_wp->fheight);
       return leNIL;
     }
 
-  newwp = window_new ();
-  newwp->fwidth = cur_wp->fwidth;
-  newwp->ewidth = cur_wp->ewidth;
+  Window *newwp = (Window *) XZALLOC (Window);
+  *newwp = *cur_wp;
   newwp->fheight = cur_wp->fheight / 2 + cur_wp->fheight % 2;
   newwp->eheight = newwp->fheight - 1;
+  newwp->saved_pt = point_marker ();
+
+  cur_wp->next = newwp;
   cur_wp->fheight = cur_wp->fheight / 2;
   cur_wp->eheight = cur_wp->fheight - 1;
   if (cur_wp->topdelta >= cur_wp->eheight)
     recenter (cur_wp);
-  newwp->bp = cur_wp->bp;
-  newwp->saved_pt = point_marker ();
-  newwp->next = cur_wp->next;
-  cur_wp->next = newwp;
 }
 END_DEFUN
 
@@ -135,9 +122,7 @@ delete_window (Window * del_wp)
     }
 
   if (del_wp->saved_pt)
-    free_marker (del_wp->saved_pt);
-
-  free (del_wp);
+    unchain_marker (del_wp->saved_pt);
 }
 
 DEFUN ("delete-window", delete_window)
@@ -160,13 +145,11 @@ DEFUN ("enlarge-window", enlarge_window)
 Make current window one line bigger.
 +*/
 {
-  Window *wp;
-
   if (cur_wp == head_wp && (cur_wp->next == NULL ||
                             cur_wp->next->fheight < 3))
     return leNIL;
 
-  wp = cur_wp->next;
+  Window *wp = cur_wp->next;
   if (wp == NULL || wp->fheight < 3)
     for (wp = head_wp; wp != NULL; wp = wp->next)
       if (wp->next == cur_wp)
@@ -190,18 +173,15 @@ DEFUN ("shrink-window", shrink_window)
 Make current window one line smaller.
 +*/
 {
-  Window *wp;
 
   if ((cur_wp == head_wp && cur_wp->next == NULL) || cur_wp->fheight < 3)
     return leNIL;
 
-  wp = cur_wp->next;
+  Window *wp = cur_wp->next;
   if (wp == NULL)
-    {
-      for (wp = head_wp; wp != NULL; wp = wp->next)
-        if (wp->next == cur_wp)
-          break;
-    }
+    for (wp = head_wp; wp != NULL; wp = wp->next)
+      if (wp->next == cur_wp)
+        break;
 
   ++wp->fheight;
   ++wp->eheight;
@@ -231,9 +211,7 @@ DEFUN ("delete-other-windows", delete_other_windows)
 Make the selected window fill the screen.
 +*/
 {
-  Window *wp, *nextwp;
-
-  for (wp = head_wp; wp != NULL; wp = nextwp)
+  for (Window *wp = head_wp, *nextwp; wp != NULL; wp = nextwp)
     {
       nextwp = wp->next;
       if (wp != cur_wp)
@@ -260,10 +238,8 @@ END_DEFUN
 void
 create_scratch_window (void)
 {
-  Window *wp;
   Buffer *bp = create_scratch_buffer ();
-
-  wp = window_new ();
+  Window *wp = (Window *) XZALLOC (Window);
   cur_wp = head_wp = wp;
   wp->fwidth = wp->ewidth = term_width ();
   /* Save space for minibuffer. */
@@ -276,17 +252,15 @@ create_scratch_window (void)
 Window *
 find_window (const char *name)
 {
-  Window *wp;
-
-  for (wp = head_wp; wp != NULL; wp = wp->next)
+  for (Window *wp = head_wp; wp != NULL; wp = wp->next)
     if (STREQ (get_buffer_name (wp->bp), name))
       return wp;
 
   return NULL;
 }
 
-Point
-window_pt (Window * wp)
+size_t
+window_o (Window * wp)
 {
   /* The current window uses the current buffer point; all other
      windows have a saved point, except that if a window has just been
@@ -302,7 +276,7 @@ window_pt (Window * wp)
   else
     {
       if (wp->saved_pt != NULL)
-        return get_marker_pt (wp->saved_pt);
+        return get_marker_o (wp->saved_pt);
       else
         return get_buffer_pt (wp->bp);
     }
@@ -311,12 +285,11 @@ window_pt (Window * wp)
 bool
 window_top_visible (Window * wp)
 {
-  return window_pt (wp).n == get_window_topdelta (wp);
+  return offset_to_line (get_window_bp (wp), window_o (wp)) == get_window_topdelta (wp);
 }
 
 bool
 window_bottom_visible (Window * wp)
 {
-  return window_pt (wp).n + (get_window_eheight (wp) - get_window_topdelta (wp)) >
-    get_buffer_last_line (get_window_bp (wp));
+  return get_window_all_displayed (wp);
 }
